# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.2.1

import enum

import winrt.system
from winrt._winrt_windows_media_casting import (
    CastingConnection,
    CastingConnectionErrorOccurredEventArgs,
    CastingDevice,
    CastingDevicePicker,
    CastingDevicePickerFilter,
    CastingDeviceSelectedEventArgs,
    CastingSource,
)

__all__ = [
    "CastingConnectionErrorStatus",
    "CastingConnectionState",
    "CastingPlaybackTypes",
    "CastingConnection",
    "CastingConnectionErrorOccurredEventArgs",
    "CastingDevice",
    "CastingDevicePicker",
    "CastingDevicePickerFilter",
    "CastingDeviceSelectedEventArgs",
    "CastingSource",
]

class CastingConnectionErrorStatus(enum.IntEnum):
    SUCCEEDED = 0
    DEVICE_DID_NOT_RESPOND = 1
    DEVICE_ERROR = 2
    DEVICE_LOCKED = 3
    PROTECTED_PLAYBACK_FAILED = 4
    INVALID_CASTING_SOURCE = 5
    UNKNOWN = 6

class CastingConnectionState(enum.IntEnum):
    DISCONNECTED = 0
    CONNECTED = 1
    RENDERING = 2
    DISCONNECTING = 3
    CONNECTING = 4

class CastingPlaybackTypes(enum.IntFlag):
    NONE = 0x0
    AUDIO = 0x1
    VIDEO = 0x2
    PICTURE = 0x4

